package at.ac.uibk.gitsearch.service;

import at.ac.uibk.gitsearch.domain.SavedSearches;
import at.ac.uibk.gitsearch.repository.SavedSearchesRepository;
import at.ac.uibk.gitsearch.repository.search.SavedSearchesSearchRepository;
import at.ac.uibk.gitsearch.service.dto.SavedSearchesDTO;
import at.ac.uibk.gitsearch.service.mapper.SavedSearchesMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Optional;

import static org.elasticsearch.index.query.QueryBuilders.*;

/**
 * Service Implementation for managing {@link SavedSearches}.
 */
@Service
@Transactional
public class SavedSearchesService {

    private final Logger log = LoggerFactory.getLogger(SavedSearchesService.class);

    private final SavedSearchesRepository savedSearchesRepository;

    private final SavedSearchesMapper savedSearchesMapper;

    private final SavedSearchesSearchRepository savedSearchesSearchRepository;

    public SavedSearchesService(SavedSearchesRepository savedSearchesRepository, SavedSearchesMapper savedSearchesMapper, SavedSearchesSearchRepository savedSearchesSearchRepository) {
        this.savedSearchesRepository = savedSearchesRepository;
        this.savedSearchesMapper = savedSearchesMapper;
        this.savedSearchesSearchRepository = savedSearchesSearchRepository;
    }

    /**
     * Save a savedSearches.
     *
     * @param savedSearchesDTO the entity to save.
     * @return the persisted entity.
     */
    public SavedSearchesDTO save(SavedSearchesDTO savedSearchesDTO) {
        log.debug("Request to save SavedSearches : {}", savedSearchesDTO);
        SavedSearches savedSearches = savedSearchesMapper.toEntity(savedSearchesDTO);
        savedSearches = savedSearchesRepository.save(savedSearches);
        SavedSearchesDTO result = savedSearchesMapper.toDto(savedSearches);
        savedSearchesSearchRepository.save(savedSearches);
        return result;
    }

    /**
     * Get all the savedSearches.
     *
     * @param pageable the pagination information.
     * @return the list of entities.
     */
    @Transactional(readOnly = true)
    public Page<SavedSearchesDTO> findAll(Pageable pageable) {
        log.debug("Request to get all SavedSearches");
        return savedSearchesRepository.findAll(pageable)
            .map(savedSearchesMapper::toDto);
    }


    /**
     * Get one savedSearches by id.
     *
     * @param id the id of the entity.
     * @return the entity.
     */
    @Transactional(readOnly = true)
    public Optional<SavedSearchesDTO> findOne(Long id) {
        log.debug("Request to get SavedSearches : {}", id);
        return savedSearchesRepository.findById(id)
            .map(savedSearchesMapper::toDto);
    }

    /**
     * Delete the savedSearches by id.
     *
     * @param id the id of the entity.
     */
    public void delete(Long id) {
        log.debug("Request to delete SavedSearches : {}", id);
        savedSearchesRepository.deleteById(id);
        savedSearchesSearchRepository.deleteById(id);
    }

    /**
     * Search for the savedSearches corresponding to the query.
     *
     * @param query the query of the search.
     * @param pageable the pagination information.
     * @return the list of entities.
     */
    @Transactional(readOnly = true)
    public Page<SavedSearchesDTO> search(String query, Pageable pageable) {
        log.debug("Request to search for a page of SavedSearches for query {}", query);
        return savedSearchesSearchRepository.search(queryStringQuery(query), pageable)
            .map(savedSearchesMapper::toDto);
    }
}
