package at.ac.uibk.gitsearch.web.rest;

import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.csrf;

import at.ac.uibk.gitsearch.GitsearchApp;
import at.ac.uibk.gitsearch.domain.Likes;
import at.ac.uibk.gitsearch.repository.LikesRepository;
import at.ac.uibk.gitsearch.repository.search.LikesSearchRepository;
import at.ac.uibk.gitsearch.service.LikesService;
import at.ac.uibk.gitsearch.service.dto.LikesCriteria;
import at.ac.uibk.gitsearch.service.LikesQueryService;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.transaction.annotation.Transactional;
import javax.persistence.EntityManager;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.Collections;
import java.util.List;

import static org.assertj.core.api.Assertions.assertThat;
import static org.elasticsearch.index.query.QueryBuilders.queryStringQuery;
import static org.hamcrest.Matchers.hasItem;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;
import at.ac.uibk.gitsearch.security.AuthoritiesConstants;

/**
 * Integration tests for the {@link LikesResource} REST controller.
 */
@SpringBootTest(classes = GitsearchApp.class)
@ExtendWith(MockitoExtension.class)
@AutoConfigureMockMvc
@WithMockUser(authorities = AuthoritiesConstants.ADMIN)
public class LikesResourceIT {

    private static final LocalDate DEFAULT_DATE = LocalDate.ofEpochDay(0L);
    private static final LocalDate UPDATED_DATE = LocalDate.now(ZoneId.systemDefault());
    private static final LocalDate SMALLER_DATE = LocalDate.ofEpochDay(-1L);

    private static final Integer DEFAULT_USER_ID = 1;
    private static final Integer UPDATED_USER_ID = 2;
    private static final Integer SMALLER_USER_ID = 1 - 1;

    private static final String DEFAULT_EXERCISE_ID = "1";
    private static final String UPDATED_EXERCISE_ID = "2";
    private static final String SMALLER_EXERCISE_ID = "0";

    @Autowired
    private LikesRepository likesRepository;

    @Autowired
    private LikesService likesService;

    /**
     * This repository is mocked in the at.ac.uibk.gitsearch.repository.search test
     * package.
     *
     * @see at.ac.uibk.gitsearch.repository.search.LikesSearchRepositoryMockConfiguration
     */
    @Autowired
    private LikesSearchRepository mockLikesSearchRepository;

    @Autowired
    private LikesQueryService likesQueryService;

    @Autowired
    private EntityManager em;

    @Autowired
    private MockMvc restLikesMockMvc;

    private Likes likes;

    /**
     * Create an entity for this test.
     *
     * This is a static method, as tests for other entities might also need it, if
     * they test an entity which requires the current entity.
     */
    public static Likes createEntity(EntityManager em) {
        Likes likes = new Likes().date(DEFAULT_DATE).userID(DEFAULT_USER_ID).exerciseID(DEFAULT_EXERCISE_ID);
        return likes;
    }

    /**
     * Create an updated entity for this test.
     *
     * This is a static method, as tests for other entities might also need it, if
     * they test an entity which requires the current entity.
     */
    public static Likes createUpdatedEntity(EntityManager em) {
        Likes likes = new Likes().date(UPDATED_DATE).userID(UPDATED_USER_ID).exerciseID(UPDATED_EXERCISE_ID);
        return likes;
    }

    @BeforeEach
    public void initTest() {
        likes = createEntity(em);
    }

    @Test
    @Transactional
    public void createLikes() throws Exception {
        int databaseSizeBeforeCreate = likesRepository.findAll().size();
        // Create the Likes
        restLikesMockMvc.perform(post("/api/likes").with(csrf().asHeader()).contentType(MediaType.APPLICATION_JSON)
                .content(TestUtil.convertObjectToJsonBytes(likes))).andExpect(status().isCreated());

        // Validate the Likes in the database
        List<Likes> likesList = likesRepository.findAll();
        assertThat(likesList).hasSize(databaseSizeBeforeCreate + 1);
        Likes testLikes = likesList.get(likesList.size() - 1);
        assertThat(testLikes.getDate()).isEqualTo(DEFAULT_DATE);
        assertThat(testLikes.getUserID()).isEqualTo(DEFAULT_USER_ID);
        assertThat(testLikes.getExerciseID()).isEqualTo(DEFAULT_EXERCISE_ID);

        // Validate the Likes in Elasticsearch
        verify(mockLikesSearchRepository, times(1)).save(testLikes);
    }

    @Test
    @Transactional
    public void createLikesWithExistingId() throws Exception {
        int databaseSizeBeforeCreate = likesRepository.findAll().size();

        // Create the Likes with an existing ID
        likes.setId(1L);

        // An entity with an existing ID cannot be created, so this API call must fail
        restLikesMockMvc.perform(post("/api/likes").with(csrf().asHeader()).contentType(MediaType.APPLICATION_JSON)
                .content(TestUtil.convertObjectToJsonBytes(likes))).andExpect(status().isBadRequest());

        // Validate the Likes in the database
        List<Likes> likesList = likesRepository.findAll();
        assertThat(likesList).hasSize(databaseSizeBeforeCreate);

        // Validate the Likes in Elasticsearch
        verify(mockLikesSearchRepository, times(0)).save(likes);
    }

    @Test
    @Transactional
    public void checkDateIsRequired() throws Exception {
        int databaseSizeBeforeTest = likesRepository.findAll().size();
        // set the field null
        likes.setDate(null);

        // Create the Likes, which fails.

        restLikesMockMvc.perform(post("/api/likes").with(csrf().asHeader()).contentType(MediaType.APPLICATION_JSON)
                .content(TestUtil.convertObjectToJsonBytes(likes))).andExpect(status().isBadRequest());

        List<Likes> likesList = likesRepository.findAll();
        assertThat(likesList).hasSize(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    public void checkUserIDIsRequired() throws Exception {
        int databaseSizeBeforeTest = likesRepository.findAll().size();
        // set the field null
        likes.setUserID(null);

        // Create the Likes, which fails.

        restLikesMockMvc.perform(post("/api/likes").with(csrf().asHeader()).contentType(MediaType.APPLICATION_JSON)
                .content(TestUtil.convertObjectToJsonBytes(likes))).andExpect(status().isBadRequest());

        List<Likes> likesList = likesRepository.findAll();
        assertThat(likesList).hasSize(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    public void checkexerciseIDIsRequired() throws Exception {
        int databaseSizeBeforeTest = likesRepository.findAll().size();
        // set the field null
        likes.setExerciseID(null);

        // Create the Likes, which fails.

        restLikesMockMvc.perform(post("/api/likes").with(csrf().asHeader()).contentType(MediaType.APPLICATION_JSON)
                .content(TestUtil.convertObjectToJsonBytes(likes))).andExpect(status().isBadRequest());

        List<Likes> likesList = likesRepository.findAll();
        assertThat(likesList).hasSize(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    public void getAllLikes() throws Exception {
        // Initialize the database
        likesRepository.saveAndFlush(likes);

        // Get all the likesList
        restLikesMockMvc.perform(get("/api/likes?sort=id,desc")).andExpect(status().isOk())
                .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
                .andExpect(jsonPath("$.[*].id").value(hasItem(likes.getId().intValue())))
                .andExpect(jsonPath("$.[*].date").value(hasItem(DEFAULT_DATE.toString())))
                .andExpect(jsonPath("$.[*].userID").value(hasItem(DEFAULT_USER_ID)))
                .andExpect(jsonPath("$.[*].exerciseID").value(hasItem(DEFAULT_EXERCISE_ID)));
    }

    @Test
    @Transactional
    public void getLikes() throws Exception {
        // Initialize the database
        likesRepository.saveAndFlush(likes);

        // Get the likes
        restLikesMockMvc.perform(get("/api/likes/{id}", likes.getId())).andExpect(status().isOk())
                .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
                .andExpect(jsonPath("$.id").value(likes.getId().intValue()))
                .andExpect(jsonPath("$.date").value(DEFAULT_DATE.toString()))
                .andExpect(jsonPath("$.userID").value(DEFAULT_USER_ID))
                .andExpect(jsonPath("$.exerciseID").value(DEFAULT_EXERCISE_ID));
    }

    @Test
    @Transactional
    public void getLikesByExerciseID() throws Exception {
        // Initialize the database
        likesRepository.saveAndFlush(likes);

        // Get the likes
        restLikesMockMvc.perform(get("/api/likes/numberOfLikes/{exerciseID}", likes.getExerciseID())).andExpect(status().isOk());

        assertEquals(1, likesRepository.findNumberOfLikesByExerciseID(likes.getExerciseID()));
    }

    @Test
    @Transactional
    public void testLikeWorkflowWithAuthorities() throws Exception {
        // Initialize the database
        likesRepository.saveAndFlush(likes);

        // generate new like
        Likes newLike = new Likes();
        newLike.setExerciseID("1000/abc/bd");

        //user likes a project

        restLikesMockMvc.perform(put("/api/likes/likeExercise").with(csrf().asHeader()).contentType(MediaType.APPLICATION_JSON)
                .content(TestUtil.convertObjectToJsonBytes(newLike))).andExpect(status().isOk());

        assertEquals(1, likesRepository.findNumberOfLikesByExerciseID("1000/abc/bd"));

        //like again and nothing should happen

        restLikesMockMvc.perform(put("/api/likes/likeExercise").with(csrf().asHeader()).contentType(MediaType.APPLICATION_JSON)
        .content(TestUtil.convertObjectToJsonBytes(newLike))).andExpect(status().isOk());


        restLikesMockMvc.perform(get("/api/likes/numberOfLikes/{exerciseID}", "1000/abc/bd").with(csrf().asHeader()).contentType(MediaType.APPLICATION_JSON)).andExpect(status().isOk());

        assertEquals(1, likesRepository.findNumberOfLikesByExerciseID("1000/abc/bd"));

        //unlike project 

        restLikesMockMvc.perform(delete("/api/likes/unlikeExercise/{exerciseID}", "1000/abc/bd").with(csrf().asHeader()).accept(MediaType.APPLICATION_JSON))
        .andExpect(status().isNoContent());

        assertEquals(0, likesRepository.findNumberOfLikesByExerciseID("1000/abc/bd"));

    }

    @Test
    @Transactional
    public void testLikeWorkflowWithOutAuthorities() throws Exception {
        // Initialize the database
        likesRepository.saveAndFlush(likes);

        // generate new like
        Likes newLike = new Likes();
        newLike.setExerciseID("1000");

        //user likes a project and nothing should happen

        restLikesMockMvc.perform(put("/api/likes/likeExercise").contentType(MediaType.APPLICATION_JSON)
                .content(TestUtil.convertObjectToJsonBytes(newLike))).andExpect(status().isForbidden());

        assertEquals(0, likesRepository.findNumberOfLikesByExerciseID("1000"));


        //unlike project 

        restLikesMockMvc.perform(delete("/api/likes/unlikeExercise/{exerciseID}", "1000").accept(MediaType.APPLICATION_JSON))
        .andExpect(status().isForbidden());

        assertEquals(0, likesRepository.findNumberOfLikesByExerciseID("1000"));

    }

    @Test
    @Transactional
    public void getLikesByIdFiltering() throws Exception {
        // Initialize the database
        likesRepository.saveAndFlush(likes);

        Long id = likes.getId();

        defaultLikesShouldBeFound("id.equals=" + id);
        defaultLikesShouldNotBeFound("id.notEquals=" + id);

        defaultLikesShouldBeFound("id.greaterThanOrEqual=" + id);
        defaultLikesShouldNotBeFound("id.greaterThan=" + id);

        defaultLikesShouldBeFound("id.lessThanOrEqual=" + id);
        defaultLikesShouldNotBeFound("id.lessThan=" + id);
    }

    @Test
    @Transactional
    public void getAllLikesByDateIsEqualToSomething() throws Exception {
        // Initialize the database
        likesRepository.saveAndFlush(likes);

        // Get all the likesList where date equals to DEFAULT_DATE
        defaultLikesShouldBeFound("date.equals=" + DEFAULT_DATE);

        // Get all the likesList where date equals to UPDATED_DATE
        defaultLikesShouldNotBeFound("date.equals=" + UPDATED_DATE);
    }

    @Test
    @Transactional
    public void getAllLikesByDateIsNotEqualToSomething() throws Exception {
        // Initialize the database
        likesRepository.saveAndFlush(likes);

        // Get all the likesList where date not equals to DEFAULT_DATE
        defaultLikesShouldNotBeFound("date.notEquals=" + DEFAULT_DATE);

        // Get all the likesList where date not equals to UPDATED_DATE
        defaultLikesShouldBeFound("date.notEquals=" + UPDATED_DATE);
    }

    @Test
    @Transactional
    public void getAllLikesByDateIsInShouldWork() throws Exception {
        // Initialize the database
        likesRepository.saveAndFlush(likes);

        // Get all the likesList where date in DEFAULT_DATE or UPDATED_DATE
        defaultLikesShouldBeFound("date.in=" + DEFAULT_DATE + "," + UPDATED_DATE);

        // Get all the likesList where date equals to UPDATED_DATE
        defaultLikesShouldNotBeFound("date.in=" + UPDATED_DATE);
    }

    @Test
    @Transactional
    public void getAllLikesByDateIsNullOrNotNull() throws Exception {
        // Initialize the database
        likesRepository.saveAndFlush(likes);

        // Get all the likesList where date is not null
        defaultLikesShouldBeFound("date.specified=true");

        // Get all the likesList where date is null
        defaultLikesShouldNotBeFound("date.specified=false");
    }

    @Test
    @Transactional
    public void getAllLikesByDateIsGreaterThanOrEqualToSomething() throws Exception {
        // Initialize the database
        likesRepository.saveAndFlush(likes);

        // Get all the likesList where date is greater than or equal to DEFAULT_DATE
        defaultLikesShouldBeFound("date.greaterThanOrEqual=" + DEFAULT_DATE);

        // Get all the likesList where date is greater than or equal to UPDATED_DATE
        defaultLikesShouldNotBeFound("date.greaterThanOrEqual=" + UPDATED_DATE);
    }

    @Test
    @Transactional
    public void getAllLikesByDateIsLessThanOrEqualToSomething() throws Exception {
        // Initialize the database
        likesRepository.saveAndFlush(likes);

        // Get all the likesList where date is less than or equal to DEFAULT_DATE
        defaultLikesShouldBeFound("date.lessThanOrEqual=" + DEFAULT_DATE);

        // Get all the likesList where date is less than or equal to SMALLER_DATE
        defaultLikesShouldNotBeFound("date.lessThanOrEqual=" + SMALLER_DATE);
    }

    @Test
    @Transactional
    public void getAllLikesByDateIsLessThanSomething() throws Exception {
        // Initialize the database
        likesRepository.saveAndFlush(likes);

        // Get all the likesList where date is less than DEFAULT_DATE
        defaultLikesShouldNotBeFound("date.lessThan=" + DEFAULT_DATE);

        // Get all the likesList where date is less than UPDATED_DATE
        defaultLikesShouldBeFound("date.lessThan=" + UPDATED_DATE);
    }

    @Test
    @Transactional
    public void getAllLikesByDateIsGreaterThanSomething() throws Exception {
        // Initialize the database
        likesRepository.saveAndFlush(likes);

        // Get all the likesList where date is greater than DEFAULT_DATE
        defaultLikesShouldNotBeFound("date.greaterThan=" + DEFAULT_DATE);

        // Get all the likesList where date is greater than SMALLER_DATE
        defaultLikesShouldBeFound("date.greaterThan=" + SMALLER_DATE);
    }

    @Test
    @Transactional
    public void getAllLikesByUserIDIsEqualToSomething() throws Exception {
        // Initialize the database
        likesRepository.saveAndFlush(likes);

        // Get all the likesList where userID equals to DEFAULT_USER_ID
        defaultLikesShouldBeFound("userID.equals=" + DEFAULT_USER_ID);

        // Get all the likesList where userID equals to UPDATED_USER_ID
        defaultLikesShouldNotBeFound("userID.equals=" + UPDATED_USER_ID);
    }

    @Test
    @Transactional
    public void getAllLikesByUserIDIsNotEqualToSomething() throws Exception {
        // Initialize the database
        likesRepository.saveAndFlush(likes);

        // Get all the likesList where userID not equals to DEFAULT_USER_ID
        defaultLikesShouldNotBeFound("userID.notEquals=" + DEFAULT_USER_ID);

        // Get all the likesList where userID not equals to UPDATED_USER_ID
        defaultLikesShouldBeFound("userID.notEquals=" + UPDATED_USER_ID);
    }

    @Test
    @Transactional
    public void getAllLikesByUserIDIsInShouldWork() throws Exception {
        // Initialize the database
        likesRepository.saveAndFlush(likes);

        // Get all the likesList where userID in DEFAULT_USER_ID or UPDATED_USER_ID
        defaultLikesShouldBeFound("userID.in=" + DEFAULT_USER_ID + "," + UPDATED_USER_ID);

        // Get all the likesList where userID equals to UPDATED_USER_ID
        defaultLikesShouldNotBeFound("userID.in=" + UPDATED_USER_ID);
    }

    @Test
    @Transactional
    public void getAllLikesByUserIDIsNullOrNotNull() throws Exception {
        // Initialize the database
        likesRepository.saveAndFlush(likes);

        // Get all the likesList where userID is not null
        defaultLikesShouldBeFound("userID.specified=true");

        // Get all the likesList where userID is null
        defaultLikesShouldNotBeFound("userID.specified=false");
    }

    @Test
    @Transactional
    public void getAllLikesByUserIDIsGreaterThanOrEqualToSomething() throws Exception {
        // Initialize the database
        likesRepository.saveAndFlush(likes);

        // Get all the likesList where userID is greater than or equal to
        // DEFAULT_USER_ID
        defaultLikesShouldBeFound("userID.greaterThanOrEqual=" + DEFAULT_USER_ID);

        // Get all the likesList where userID is greater than or equal to
        // UPDATED_USER_ID
        defaultLikesShouldNotBeFound("userID.greaterThanOrEqual=" + UPDATED_USER_ID);
    }

    @Test
    @Transactional
    public void getAllLikesByUserIDIsLessThanOrEqualToSomething() throws Exception {
        // Initialize the database
        likesRepository.saveAndFlush(likes);

        // Get all the likesList where userID is less than or equal to DEFAULT_USER_ID
        defaultLikesShouldBeFound("userID.lessThanOrEqual=" + DEFAULT_USER_ID);

        // Get all the likesList where userID is less than or equal to SMALLER_USER_ID
        defaultLikesShouldNotBeFound("userID.lessThanOrEqual=" + SMALLER_USER_ID);
    }

    @Test
    @Transactional
    public void getAllLikesByUserIDIsLessThanSomething() throws Exception {
        // Initialize the database
        likesRepository.saveAndFlush(likes);

        // Get all the likesList where userID is less than DEFAULT_USER_ID
        defaultLikesShouldNotBeFound("userID.lessThan=" + DEFAULT_USER_ID);

        // Get all the likesList where userID is less than UPDATED_USER_ID
        defaultLikesShouldBeFound("userID.lessThan=" + UPDATED_USER_ID);
    }

    @Test
    @Transactional
    public void getAllLikesByUserIDIsGreaterThanSomething() throws Exception {
        // Initialize the database
        likesRepository.saveAndFlush(likes);

        // Get all the likesList where userID is greater than DEFAULT_USER_ID
        defaultLikesShouldNotBeFound("userID.greaterThan=" + DEFAULT_USER_ID);

        // Get all the likesList where userID is greater than SMALLER_USER_ID
        defaultLikesShouldBeFound("userID.greaterThan=" + SMALLER_USER_ID);
    }

    /**
     * Executes the search, and checks that the default entity is returned.
     */
    private void defaultLikesShouldBeFound(String filter) throws Exception {
        restLikesMockMvc.perform(get("/api/likes?sort=id,desc&" + filter)).andExpect(status().isOk())
                .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
                .andExpect(jsonPath("$.[*].id").value(hasItem(likes.getId().intValue())))
                .andExpect(jsonPath("$.[*].date").value(hasItem(DEFAULT_DATE.toString())))
                .andExpect(jsonPath("$.[*].userID").value(hasItem(DEFAULT_USER_ID)))
                .andExpect(jsonPath("$.[*].exerciseID").value(hasItem(DEFAULT_EXERCISE_ID)));

        // Check, that the count call also returns 1
        restLikesMockMvc.perform(get("/api/likes/count?sort=id,desc&" + filter)).andExpect(status().isOk())
                .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE)).andExpect(content().string("1"));
    }

    /**
     * Executes the search, and checks that the default entity is not returned.
     */
    private void defaultLikesShouldNotBeFound(String filter) throws Exception {
        restLikesMockMvc.perform(get("/api/likes?sort=id,desc&" + filter)).andExpect(status().isOk())
                .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE)).andExpect(jsonPath("$").isArray())
                .andExpect(jsonPath("$").isEmpty());

        // Check, that the count call also returns 0
        restLikesMockMvc.perform(get("/api/likes/count?sort=id,desc&" + filter)).andExpect(status().isOk())
                .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE)).andExpect(content().string("0"));
    }

    @Test
    @Transactional
    public void getNonExistingLikes() throws Exception {
        // Get the likes
        restLikesMockMvc.perform(get("/api/likes/{id}", Long.MAX_VALUE)).andExpect(status().isNotFound());
    }

    @Test
    @Transactional
    public void updateLikes() throws Exception {
        // Initialize the database
        likesService.save(likes);

        int databaseSizeBeforeUpdate = likesRepository.findAll().size();

        // Update the likes
        Likes updatedLikes = likesRepository.findById(likes.getId()).get();
        // Disconnect from session so that the updates on updatedLikes are not directly
        // saved in db
        em.detach(updatedLikes);
        updatedLikes.date(UPDATED_DATE).userID(UPDATED_USER_ID).exerciseID(UPDATED_EXERCISE_ID);

        restLikesMockMvc.perform(put("/api/likes").with(csrf().asHeader()).contentType(MediaType.APPLICATION_JSON)
                .content(TestUtil.convertObjectToJsonBytes(updatedLikes))).andExpect(status().isOk());

        // Validate the Likes in the database
        List<Likes> likesList = likesRepository.findAll();
        assertThat(likesList).hasSize(databaseSizeBeforeUpdate);
        Likes testLikes = likesList.get(likesList.size() - 1);
        assertThat(testLikes.getDate()).isEqualTo(UPDATED_DATE);
        assertThat(testLikes.getUserID()).isEqualTo(UPDATED_USER_ID);
        assertThat(testLikes.getExerciseID()).isEqualTo(UPDATED_EXERCISE_ID);

        // Validate the Likes in Elasticsearch
        verify(mockLikesSearchRepository, times(2)).save(testLikes);
    }

    @Test
    @Transactional
    public void updateNonExistingLikes() throws Exception {
        int databaseSizeBeforeUpdate = likesRepository.findAll().size();

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restLikesMockMvc.perform(put("/api/likes").with(csrf().asHeader()).contentType(MediaType.APPLICATION_JSON)
                .content(TestUtil.convertObjectToJsonBytes(likes))).andExpect(status().isBadRequest());

        // Validate the Likes in the database
        List<Likes> likesList = likesRepository.findAll();
        assertThat(likesList).hasSize(databaseSizeBeforeUpdate);

        // Validate the Likes in Elasticsearch
        verify(mockLikesSearchRepository, times(0)).save(likes);
    }

    @Test
    @Transactional
    public void deleteLikes() throws Exception {
        // Initialize the database
        likesService.save(likes);

        int databaseSizeBeforeDelete = likesRepository.findAll().size();

        // Delete the likes
        restLikesMockMvc.perform(
                delete("/api/likes/{id}", likes.getId()).with(csrf().asHeader()).accept(MediaType.APPLICATION_JSON))
                .andExpect(status().isNoContent());

        // Validate the database contains one less item
        List<Likes> likesList = likesRepository.findAll();
        assertThat(likesList).hasSize(databaseSizeBeforeDelete - 1);

        // Validate the Likes in Elasticsearch
        verify(mockLikesSearchRepository, times(1)).deleteById(likes.getId());
    }

    @Test
    @Transactional
    public void searchLikes() throws Exception {
        // Configure the mock search repository
        // Initialize the database
        likesService.save(likes);
        when(mockLikesSearchRepository.search(queryStringQuery("id:" + likes.getId())))
                .thenReturn(Collections.singletonList(likes));

        // Search the likes
        restLikesMockMvc.perform(get("/api/_search/likes?query=id:" + likes.getId())).andExpect(status().isOk())
                .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
                .andExpect(jsonPath("$.[*].id").value(hasItem(likes.getId().intValue())))
                .andExpect(jsonPath("$.[*].date").value(hasItem(DEFAULT_DATE.toString())))
                .andExpect(jsonPath("$.[*].userID").value(hasItem(DEFAULT_USER_ID)))
                .andExpect(jsonPath("$.[*].exerciseID").value(hasItem(DEFAULT_EXERCISE_ID)));
    }
}
